package stackjava.com.hibernatepagination.dao;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.*;
import org.hibernate.cfg.Configuration;

import stackjava.com.hibernatepagination.model.Customer;

public class CustomerDAO {
	SessionFactory sessionFactory = new Configuration().configure().buildSessionFactory();

	public void save(Customer customer) {
		Session session = sessionFactory.openSession();
		try {
			session.beginTransaction();
			session.save(customer);
			session.getTransaction().commit();
			System.out.println("insert success!");
		} catch (RuntimeException e) {
			session.getTransaction().rollback();
			e.printStackTrace();
		} finally {
			session.flush();
			session.close();
		}
	}
	
	public long countTotalRecords() {
		Session session = sessionFactory.openSession();
		String countQ = "Select count (c.id) from Customer c";
		Query countQuery = session.createQuery(countQ);
		return (Long) countQuery.uniqueResult();
	}
	
	public List<Customer> getUsingSession(int position, int pageSize) {
		Session session = sessionFactory.openSession();
		Query query = session.createQuery("FROM Customer");
		query.setFirstResult(position);
		query.setMaxResults(pageSize);
		return query.list();
	}
	
	public List<Customer> getUsingCriteria(int position, int pageSize) {
		Session session = sessionFactory.openSession();
		Criteria criteria = session.createCriteria(Customer.class);
		criteria.setFirstResult(position);
		criteria.setMaxResults(pageSize);
		return criteria.list();
	}
	
	public List<Customer> getUsingScroll(int position, int pageSize) {
		Session session = sessionFactory.openSession();
		Query query = session.createQuery("FROM Customer");
		 
		ScrollableResults resultScroll = query.scroll(ScrollMode.FORWARD_ONLY);
		resultScroll.setRowNumber(position);
		resultScroll.scroll(0);
		List<Customer> listCustomer = new ArrayList<Customer>();
		int i = 0;
		while (pageSize > i) {
			listCustomer.add((Customer) resultScroll.get(0));
		    if (!resultScroll.next()) {
		    	break;
		    }
		    i++;
		}
		return listCustomer;
	}
	
}
